/* Sunburst Styles for Seantis Dir Events */

/* directory search header */
#directorySearch {
    background-color: #efefef;
    border: none;
    border-radius: 3px;
}

#directorySearch label {
    color: #555;
    font-weight: normal   
}

/* header description */
#landingPageIntro {
    margin: 10px 0 0 0;
    font-size: 1.2em;  
}

/* directory content block */
#directoryInfo {
    margin: 10px 0 0 0;   
}

/* event directory columns */
.event-navigation, #directoryResultList, .event-sidebar, .event-view {
    float: left;
    box-sizing: border-box;
}

.event-navigation, #directoryResultList {
    padding-right: 25px;
}

.event-navigation {
    width: 20%;
}

#directoryResultList {
    width: 45%;
}

.event-sidebar {
    width: 35%;   
}

.event-view {
    width: 65%;   
}

@media (max-width: 900px) {
    .event-navigation, #directoryResultList, .event-sidebar, .event-view {
         float: none;
         width: 100%;
         padding: 0;  
    }
}

/* event navigation */
.event-navigation h3 {
    color: black;
    margin-bottom: 2px;
}

.event-navigation ul {
    margin: 0 !important;
    margin-bottom: 5px !important;
}

.event-navigation ul li {
    margin: 0 !important;
    list-style: none;
}

/* event navigation links */
.event-navigation ul a {
    border-bottom: none !important; 
    color: black !important;
    display: block;
    background-color: #efefef;
    padding: 2px 4px;
    border-radius: 3px;
}

#content .event-navigation ul a:hover,
.event-navigation ul a.selected {
    background-color: #205c90;
    color: white !important;
}

/* event results */
.directoryResult {
    border: none !important;
    border-bottom: 1px solid #ddd !important;
}

.directoryResult:last-child {
    border-bottom: none !important;   
}

.directoryMarker {
    right: 0;   
}

.eventgroup-title {
    margin-top: 6px;
    margin-bottom: 2px !important;
}

/* admin controls */
.event-workflow {
    float: left;
    margin-top: 5px;
    font-weight: bold;
}

.event-state {
    color: #555;   
}

.event-actions {
    font-weight: bold;   
}

/* single result */
.eventgroup-title {
    font-size: 1.4em;
    color: #205c90;
    font-weight: 100;
    text-align: right;
    border: none !important;
    border-bottom: #0383be solid !important;
    padding-bottom: 2px;
}

.event-text h3 {
    font-size: 1.5em;   
    font-weight: normal;
}

.event-text {
    color: #333;
    font-size: 12px;
}

.event-text .locality {
    font-weight: bold;
}

.event-image {
    float: right;
    margin: 7px 50px;
    border: 1px solid #ccc;
    background-color: #efefef;
    padding: 15px;
}

/* directory map */
#directoryResultMap {
    margin-top: 26px;
    margin-bottom: 26px;
}

/* tags */
#content .eventtags a,
#content .eventtags a:hover {
    border: none;
}

.eventtag {
    background-color: #efefef;
    padding: 2px 4px;
    border-radius: 3px;   
}

/* event time */
.timerange, .event-date {
    background-color: #c9f685;
    padding: 2px 4px;
    border-radius: 3px;
    margin: 0 50px 0 0;
    float: right;
    text-align: center;
    display: inline-block;
    clear: both;
    min-width: 88px;
    color: green;
}

/* submit event */
.your-event {
    font-size: 1.8em;
    margin-top: 20px;
    margin-bottom: 20px;
    padding: 30px;
    background-color: #0383be;
    color: white;
}

#content .your-event a,
#content .your-event a:hover {
    color: white;
    border-color: white;
    border-width: 0 0 4px 0;
}

/* event detail head */
.event-date {
    font-size: 1.2em;
    font-weight: bold;
    float: none;
    margin-top: 5px;
    margin-bottom: 5px;
}

.descriptions {
    max-width: 700px;
    margin-right: 50px;
}

.descriptions .short {
    font-weight: bold;
    margin: 5px 0 10px 0;   
}

.descriptions .long {
    color: #333;
    margin: 0 0 25px 0;   
}

/* adjust map on event item view */
#directoryDetails .event-sidebar {
    margin-top: 14px;   
}

/* same with tags */
#directoryDetails .eventtags {
    margin-bottom: 25px;    
}

/* event detail blocks */
.event-details > div {
    margin-bottom: 25px;
    margin-right: 50px;      
}

.event-details h3 {
    font-weight: normal;
    font-size: 1.2em;
    border-bottom: 1px solid #eee;
    margin-bottom: 10px;
    color: black;
}

.event-details ul {
    margin-left: 0 !important;
    list-style: none !important;
}

@media (max-width: 900px) {
    .event-details > div, .descriptions {
        margin-right: 0 !important;
    }
}

/* submit form */
#formfield-form-widgets-cat1,
#formfield-form-widgets-cat2 {
    width: 50%;
    float: left;
    clear: none;  
}
#formfield-form-widgets-cat1 > span,
#formfield-form-widgets-cat2 > span {
    display: block;   
}

#formfield-form-widgets-long_description textarea {
    height: 200px;
}